"use strict";

/* exported logger */

var AbstractAdminPanel = require('./AbstractAdminPanel');
var AddProjectDialog = require('./AddProjectDialog');
var EditProjectDialog = require('./EditProjectDialog');

var logger = require('../../logger');

var Functions = require('../../Functions');
var GuiConnector = require('../../GuiConnector');
var Promise = require("bluebird");

function MapsAdminPanel(params) {
    var self = this;
    AbstractAdminPanel.call(self, params);
    self._createGui();

    $(self.getElement()).addClass("minerva-projects-tab");

}

MapsAdminPanel.prototype = Object.create(AbstractAdminPanel.prototype);
MapsAdminPanel.prototype.constructor = MapsAdminPanel;

MapsAdminPanel.prototype._createGui = function () {
    var self = this;
    var projectsDiv = Functions.createElement({
        type: "div",
    });
    self.getElement().appendChild(projectsDiv);

    var dataDiv = Functions.createElement({
        type: "div",
        style: "display:table; width:100%",
    });
    projectsDiv.appendChild(dataDiv);

    dataDiv.appendChild(self._createMenuRow());
    dataDiv.appendChild(self._createProjectTableRow());
    dataDiv.appendChild(self._createMenuRow());

};

MapsAdminPanel.prototype._createMenuRow = function () {
    var self = this;
    var menuRow = Functions.createElement({
        type: "div",
        className: "minerva-menu-row",
        style: "display:table-row; margin:10px",
    });

    var addProjectButton = Functions.createElement({
        type: "button",
        name: "addProject",
        content: '<span class="ui-icon ui-icon-circle-plus"></span>&nbsp;ADD PROJECT',
        onclick: function () {
            return self.onAddClicked().then(null, GuiConnector.alert);
        },
    });
    var refreshButton = Functions.createElement({
        type: "button",
        name: "refreshProject",
        content: '<span class="ui-icon ui-icon-refresh"></span>&nbsp;REFRESH',
        onclick: function () {
            return self.onRefreshClicked().then(null, GuiConnector.alert);
        },
    });
    menuRow.appendChild(addProjectButton);
    menuRow.appendChild(refreshButton);
    return menuRow;
};

MapsAdminPanel.prototype._createProjectTableRow = function () {
    var self = this;
    var projectsRow = Functions.createElement({
        type: "div",
        style: "display:table-row; width:100%",
    });

    var projectsTable = Functions.createElement({
        type: "table",
        name: "projectsTable",
        className: "display",
        style: "width:100%",
    });
    projectsRow.appendChild(projectsTable);

    $(projectsTable).DataTable({
        fnRowCallback: function (nRow, aData) {
            nRow.setAttribute('id', aData[0]);
        },
        columns: [{
            title: 'ProjectId',
        }, {
            title: 'Name',
        }, {
            title: 'Disease',
        }, {
            title: 'Organism',
        }, {
            title: 'Status',
        }, {
            title: 'Edit',
        }, {
            title: 'Remove',
        },],
    });
    $(projectsTable).on("click", "[name='removeProject']", function () {
        var button = this;
        return self.removeProject($(button).attr("data")).then(null, GuiConnector.alert);
    });

    $(projectsTable).on("click", "[name='showEditDialog']", function () {
        var button = this;
        return self.showEditDialog($(button).attr("data")).then(null, GuiConnector.alert);
    });

    return projectsRow;
};

MapsAdminPanel.prototype.init = function () {
    var self = this;
    return ServerConnector.getProjects().then(function (projects) {
        return self.setProjects(projects);
    }).then(function () {
        $(window).trigger('resize');
    });
};

MapsAdminPanel.prototype.projectToTableRow = function (project, row) {
    var self = this;
    var disease = self.getHtmlStringLink(project.getDisease());
    var organism = self.getHtmlStringLink(project.getOrganism());

    if (row === undefined) {
        row = [];
    }
    row[0] = project.getProjectId();
    row[1] = project.getName();
    row[2] = disease;
    row[3] = organism;
    row[4] = project.getStatus();
    row[5] = "<button name='showEditDialog' data='" + project.getProjectId() + "'>EDIT</button>";
    row[6] = "<button name='removeProject' data='" + project.getProjectId() + "'>REMOVE</button>";

    return row;
};

MapsAdminPanel.prototype.getHtmlStringLink = function (annotation) {
    var self = this;
    if (annotation !== undefined && annotation !== null) {
        var link = self.getGuiUtils().createAnnotationLink(annotation, true);
        var tmp = document.createElement("div");
        tmp.appendChild(link);
        return tmp.innerHTML;
    } else {
        return "N/A";
    }

};

MapsAdminPanel.prototype.setProjects = function (projects) {
    var self = this;
    var dataTable = $($("[name='projectsTable']", self.getElement())[0]).DataTable();
    var data = [];
    for (var i = 0; i < projects.length; i++) {
        var project = projects[i];
        var rowData = self.projectToTableRow(project);
        self.addUpdateListener(project, rowData);
        data.push(rowData);
    }
    dataTable.clear().rows.add(data).draw();
};

MapsAdminPanel.prototype.addUpdateListener = function (project, dataTableRow) {
    var self = this;

    var listenerName = "PROJECT_LIST_LISTENER";
    var listeners = project.getListeners("onreload");
    for (var i = 0; i < listeners.length; i++) {
        if (listeners[i].listenerName === listenerName) {
            project.removeListener("onreload", listeners[i]);
        }
    }
    var listener = function () {
        self.projectToTableRow(project, dataTableRow);
        var row = $($("[name='projectsTable']", self.getElement())[0]).DataTable().row("#" + project.getProjectId());
        if (row.length > 0) {
            row.data(dataTableRow).draw();
        }

    };
    listener.listenerName = listenerName;
    project.addListener("onreload", listener);
};

MapsAdminPanel.prototype.onAddClicked = function () {
    var self = this;
    var dialog = self._addDialog;
    if (dialog === undefined) {
        dialog = new AddProjectDialog({
            element: Functions.createElement({
                type: "div"
            }),
            customMap: null,
        });
        self._addDialog = dialog;
        return dialog.init().then(function () {
            return dialog.open();
        });
    } else {
        dialog.clear();
        dialog.open();
        return Promise.resolve();
    }
};

MapsAdminPanel.prototype.destroy = function () {
    var self = this;
    var dialog = self._addDialog;
    if (dialog !== undefined) {
        dialog.destroy();
    }
    var table = $("[name='projectsTable']", self.getElement())[0];
    if ($.fn.DataTable.isDataTable(table)) {
        $(table).DataTable().destroy();
    }

};

MapsAdminPanel.prototype.onRefreshClicked = function () {
    var self = this;
    return ServerConnector.getProjects(true).then(function (projects) {
        return self.setProjects(projects);
    });
};

MapsAdminPanel.prototype.getDialog = function (project) {
    var self = this;
    if (self._dialogs === undefined) {
        self._dialogs = [];
    }
    var dialog = self._dialogs[project.getProjectId()];
    if (dialog === undefined) {
        dialog = new EditProjectDialog({
            element: Functions.createElement({
                type: "div"
            }),
            project: project,
            customMap: null,
        });
        self._dialogs[project.getProjectId()] = dialog;
        return dialog.init().then(function () {
            return dialog;
        });
    } else {
        return Promise.resolve(dialog);
    }
};

MapsAdminPanel.prototype.showEditDialog = function (id) {
    var self = this;
    GuiConnector.showProcessing();
    return ServerConnector.getProject(id).then(function (project) {
        return self.getDialog(project);
    }).then(function (dialog) {

        dialog.open();
        GuiConnector.hideProcessing();
    }).then(null, function (error) {
        GuiConnector.hideProcessing();
        return Promise.reject(error);
    });
};

MapsAdminPanel.prototype.removeProject = function (id) {
    return Promise.reject(new Error("Not implemented"));
};

module.exports = MapsAdminPanel;
